dojo.provide('ram.delegated.SelectionDialog');
dojo.require('dijit._Templated');
dojo.require('dijit._Widget');
dojo.require('ram.NlsMixin');
dojo.require('dojo.data.ItemFileReadStore');
dojo.require('dojox.grid.DataGrid');
dojo.require('dojo.fx.easing');

var updatedSearchResultCountTimer;

dojo.global.assetTypeUrlToNameMap = {};

dojo.global.formatAssetType = function(item) {
	if (item['dcterms:type']) {
		var typeUrl = item['dcterms:type'][0]['rdf:resource'];
		if (!dojo.global.assetTypeUrlToNameMap[typeUrl]) {
			dojo.xhrGet({
				url: typeUrl,
				headers: {
					'Accept': 'application/json',
					'OSLC-Core-Version': '2.0'
				},
				handleAs: 'json',
				sync: true,
				load: function(assetType) {
					dojo.global.assetTypeUrlToNameMap[typeUrl] = assetType.name;
				}
			});
		}
		var type = dojo.global.assetTypeUrlToNameMap[typeUrl];
		return type;
	}
	return '';
}

dojo.declare(
	'ram.delegated.SelectionDialog',
	[dijit._Widget, dijit._Templated, ram.NlsMixin],
{
	widgetsInTemplate: true,
	templateString: null,
	templatePath: dojo.moduleUrl('ram.delegated', 'templates/SelectionDialog.html'),
	dataStore: null,
	selectedFacets: [],
	filtersLabel: '',
	nameLabel: '',
	versionLabel: '',
	typeLabel: '',
	searchLabel: '',
	okLabel: '',
	cancelLabel: '',
	filter: '',
	query: '',
	
	postCreate: function() {
		try {
			this.searchInput.focus();
		} catch (e) {
			// if not visible, this could throw an exception on IE (see defect 87844)
		}
		this.filterChevronIcon.src = getAbsoluteUrl('theme/images/icons/obj16/chev-left.gif');
		this.loadingImage.src = getAbsoluteUrl('theme/images/icon_animated_busy.gif');
		var fetchFacets;
		if (this.filter != '') {
			fetchFacets = dojo.xhrGet({
				url: getAbsoluteUrl('internal/service/FacetService.getFilteredFacets?filter=' + this.filter),
				handleAs: 'json',
				load: dojo.hitch(this, function(response) {
					dojo.forEach(response, dojo.hitch(this, function(facet) {
						this.addFacet(facet, false);
					}));
					if (this.query != '') {
						this.searchInput.value = this.query;
						this.search();
					}
				})
			});
		} else {
			if (this.query != '') {
				this.searchInput.value = this.query;
				this.search();
			}
		}
	},
	
	_getUpdatedSearchResultCount: function(queryText, originalSearchResultCount, synchronizedSearch, updatedSearchResultCountChecks) {
		var self = this;
		dojo.xhrGet({
			url: getAbsoluteUrl('internal/service/AssetSearchResultCountService.getAssetSearchResultCount'),
			content: {text: queryText, synchronizedSearch: synchronizedSearch},
			handleAs: 'text',
			timeout: 30000,
			load: function(synchronizedCount, ioArgs) {
				if (synchronizedCount > originalSearchResultCount) {
					var difference = synchronizedCount - originalSearchResultCount;
					self.synchronizedSearchMessageText.innerHTML = difference + ' more results since you started searching.  ';
					if (self.synchronizedSearchMessage.style.display == 'none') {
						dojo.animateProperty({
							node: self.synchronizedSearchMessage,
							properties: {
								height: 23
							},
							easing: dojo.fx.easing.quadOut,
							duration: 1000,
							beforeBegin: function(node) {
								document.getElementById('resultsGrid').style.height = '360px';
								dojo.style(self.resultsSection, 'height', '360px');
								dojo.style(node, 'display', '');
							}
						}).play();
					}
				} else {
					dojo.style(self.synchronizedSearchMessage, 'display', 'none');
					dojo.style(self.resultsSection, 'height', '380px');
					document.getElementById('resultsGrid').style.height = '380px';
				}
				
				if (updatedSearchResultCountChecks < 20) {
					updatedSearchResultCountChecks++;
					updatedSearchResultCountTimer = setTimeout(function() { self._getUpdatedSearchResultCount(queryText, originalSearchResultCount, false, updatedSearchResultCountChecks) }, 20000);
				}
			},
			error: function(error) {
				console.error('Error checking for updated search results.  HTTP status code: ', ioArgs.xhr.status);
			}
		});
	},
	
	search: function() {
		if (updatedSearchResultCountTimer != null) {
			clearTimeout(updatedSearchResultCountTimer);
		}
		var self = this;
		if (this.searchInput.value.length > 0 || this.selectedFacets.length > 0) {
			var self = this;
			var queryText = new StringBuffer();
			queryText.append('ramSearch:(1');
			if (this.searchInput.value.length > 0) {
				queryText.append('txt%2C');
				queryText.append(encodeURIComponent(this.searchInput.value));
				if (this.selectedFacets.length > 0)
					queryText.append('%24');
			}
			for (var i = 0; i < this.selectedFacets.length; i++) {
				if (i > 0)
					queryText.append('%24');
				queryText.append(this.selectedFacets[i].facetName);
				queryText.append('%2C');
				queryText.append(this.selectedFacets[i].itemName);
			}
			queryText.append(')');
			
			dojo.style(this.loadingImage, 'display', '');
			dojo.xhrGet({
				url: getAbsoluteUrl('oslc/assets.json'),
				content: {
					'oslc_asset.query': queryText.toString(),
					'pageSize': '100'
				},
				headers: {
					'Accept': 'application/json', 
					'OSLC-Core-Version': '2.0'
				},
				handleAs: 'json',
				load: function(data) {
					if (data.results) {
						var storeData = {
							label: 'title',
							items: data.results
						};
						self.dataStore = new dojo.data.ItemFileReadStore({ data: storeData });
					} else {
						self.dataStore = null;
					}
					var grid = dijit.byId('resultsGrid');
					grid.setStore(self.dataStore);
					dojo.style(self.loadingImage, 'display', 'none');
					
					if(data) {
						queryText.append('%24sdw)');
						self._getUpdatedSearchResultCount(queryText.toString(), data.totalCount, true, 1);
					}
				},
				error: function(error) {
					dojo.style(self.loadingImage, 'display', 'none');
				}
			});
		} else {
			// clear search results
			this.dataStore = null;
			var grid = dijit.byId('resultsGrid');
			grid.setStore(null);
		}
	},
	
	searchButtonClicked: function() {
		this.search();
		this.fetchFacets();
	},
	
	searchKeyPressed: function(event) {
		if (event.keyCode == dojo.keys.ENTER) {
			this.searchButtonClicked();
		}
	},
	
	closeFilterSection: function() {
		var self = this;
		dojo.animateProperty({
			node: this.filterSection,
			properties: {
				width: 0
			},
			easing: dojo.fx.easing.quadOut,
			duration: 300,
			onEnd: function(node) {
				self.filterChevronIcon.src = getAbsoluteUrl('theme/images/icons/obj16/chev-left.gif');
				dojo.style(node, 'display', 'none');
			}
		}).play();
	},
	
	toggleFilterSection: function() {
		var self = this;
		if (dojo.style(this.filterSection, 'display') == 'none') {
			dojo.animateProperty({
				node: this.filterSection,
				properties: {
					width: 250
				},
				easing: dojo.fx.easing.quadOut,
				duration: 300,
				beforeBegin: function(node) {
					dojo.style(node, 'display', '');
					self.fetchFacets();
				},
				onEnd: function(node) {
					self.filterChevronIcon.src = getAbsoluteUrl('theme/images/icons/obj16/chev-right.gif');
				}
			}).play();
		} else {
			dojo.animateProperty({
				node: this.filterSection,
				properties: {
					width: 0
				},
				easing: dojo.fx.easing.quadOut,
				duration: 300,
				onEnd: function(node) {
					self.filterChevronIcon.src = getAbsoluteUrl('theme/images/icons/obj16/chev-left.gif');
					dojo.style(node, 'display', 'none');
				}
			}).play();
		}
	},
	
	loadFacets: function(facets) {
		this.facetDiv.innerHTML = '';
		var self = this;
		dojo.forEach(facets, function(type) {
			dojo.create('div', {'class': 'sidebarTitle', innerHTML: type.displayName}, self.facetDiv);
			var ul = dojo.create('ul', null, self.facetDiv);
			dojo.forEach(type.items, function(facet) {
				var li = dojo.create('li', null, ul);
				var a = dojo.create('a', {href: 'javascript:', innerHTML: facet.displayName + ' &lrm;[' + facet.count + ']'}, li);
				dojo.connect(a, 'onclick', facet, function() {
					self.addFacet(this);
				});
			});
		});
	},
	
	addFacet: function(facet, allowRemove) {
		this.selectedFacets.push(facet);
		dojo.style(this.selectedFacetsDiv, 'display', '');
		var span = dojo.create('span', null, this.selectedFacetsDiv);
		var self = this;
		if (allowRemove != false) {
			var a = dojo.create('a', {href: 'javascript:'}, span);
			dojo.style(a, 'textDecoration', 'none');
			dojo.connect(a, 'onclick', {facet: facet, node: span}, function() {
				self.removeFacet(this.facet, this.node);
			});
			var img = dojo.create('img', {src: getAbsoluteUrl('theme/images/icons/elcl16/delete.gif')}, a);
			dojo.style(img, 'paddingRight', '3px');
		} else {
			var img = dojo.create('img', {src: getAbsoluteUrl('theme/images/icons/elcl16/delete.gif')}, span);
			dojo.style(img, 'paddingRight', '3px');
			dojo.style(img, 'visibility', 'hidden');
		}
		dojo.create('span', {'class': 'facetName', innerHTML: facet.facetDisplayName + ': '}, span);
		dojo.create('span', {'class': 'facetSelection', innerHTML: facet.displayName}, span);
		dojo.create('br', null, span);
		if (allowRemove != false) {
			this.fetchFacets();
			this.search();
		}
	},
	
	removeFacet: function(facet, node) {
		for (var i = 0; i < this.selectedFacets.length; i++) {
			var f = this.selectedFacets[i];
			if (f.facetName == facet.facetName && f.itemName == facet.itemName) {
				this.selectedFacets.splice(i, 1);
				break;
			}
		}
		if (this.selectedFacets.length == 0)
			dojo.style(this.selectedFacetsDiv, 'display', 'none');			
		node.parentNode.removeChild(node);
		this.fetchFacets();
		this.search();
	},
	
	fetchFacets: function() {
		var self = this;
		var facetParam = new StringBuffer();
		if (this.selectedFacets.length > 0) {
			facetParam.append('&selectedFacets=');
			dojo.forEach(this.selectedFacets, function(facet) {
				facetParam.append(facet.facetName);
				facetParam.append(':');
				facetParam.append(facet.itemName);
				facetParam.append(',');
			});
		}
		dojo.xhrGet({
			url: getAbsoluteUrl('internal/service/FacetService.getFacets?query=' + self.searchInput.value + facetParam.toString()),
			handleAs: 'json',
			load: function(data) {
				self.loadFacets(data);
			},
			error: function(error) {
				alert(error);
			}
		});
	},

	sendResponse: function(response) {
		if (window.location.hash == '#oslc-core-windowName-1.0' || window.location.hash == '#oslc-windowName-1.0') {       
			   var returnURL = window.name;
			   window.name = response;
			   window.location = returnURL;
		} else if (window.location.hash == '#oslc-core-postMessage-1.0' || window.location.hash == '#oslc-postMessage-1.0') {
			(window.parent || window).postMessage('oslc-response:' + response, '*');
		}
	},

	ok: function() {
		var grid = dijit.byId('resultsGrid');
		var selectedItems = grid.selection.getSelected();
		if (selectedItems.length > 0) {
			var response = new StringBuffer('{"oslc:results" : [');
			for (var i = 0; i < selectedItems.length; i++) {
				if (i > 0) // don't insert a comma before the 1st element
					response.append(', ');
				var asset = selectedItems[i];
				response.append('{"oslc:label": "');
				response.append(asset['dcterms:title'][0]);
				response.append('","rdf:resource": "');
				var url = new StringBuffer();
				if (asset['xml:base'] && asset['xml:base'].length > 0)
					url.append(asset['xml:base'][0]);
				url.append(asset['rdf:about'][0]);
				response.append(url.toString());
				response.append('"}');
			}
			response.append(']}');
			this.sendResponse(response.toString());
		} else {
			this.cancel(); // nothing selected
		}
	},

	cancel: function() {
		var response = '{"oslc:results" : []}';
		this.sendResponse(response);
	},
	
	initializeStrings: function() {
		this.filtersLabel = this.getString('selectionDialog_Filters');
		this.nameLabel = this.getString('tableHeader_Name');
		this.versionLabel = this.getString('tableHeader_Version');
		this.typeLabel = this.getString('tableHeader_Type');
		this.searchLabel = this.getString('buttonSearch');
		this.okLabel = this.getString('buttonOk');
		this.cancelLabel = this.getString('buttonCancel');
	}
	
});
