dojo.provide("ram.relationshiptree.EditorRelationshipTree");

dojo.require("dijit.Tree");
dojo.require("dijit.Toolbar");
dojo.require("jazz.ui.RemoteChooserContainer");

dojo.require('dijit.form.TextBox');
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dijit.form.CheckBox");
dojo.require('dijit.form.Button');

dojo.require('ram.RamDialog');

dojo.requireLocalization("ram", "RelationshipTreeMessages");

dojo.declare(
		"ram.relationshiptree._EditorConstraintTreeNode",
		[dijit._TreeNode],
{
	templateString: dojo.cache("ram", "templates/EditorConstraintTreeNode.html"),
	
	insertButton: null,
	deleteButton: null,
	
	constructor: function() {
		this.msg = dojo.i18n.getLocalization("ram", "RelationshipTreeMessages");
	},
	
	postCreate:function () {
		this.inherited(arguments);
		this._createMenuBar();
		this._createConstraintInfo();
	},
	_createMenuBar: function() {     	
		var pMenuBar = new dijit.Toolbar();
		
		this.insertButton = new dijit.form.Button({
            label: this.msg.buttonInsert,
            iconClass: "dijitEditorIcon rowActionInsert",
            showLabel: false
        });
        pMenuBar.addChild(this.insertButton);
        
        pMenuBar.placeAt(this.menuBarWrapper);
	},
	_createConstraintInfo: function() {
		this.constraintNode.appendChild(dojo.doc.createTextNode(this.tree.model.store.getValue(this.item, "constraintString")));
	}
});

dojo.declare(
		"ram.relationshiptree._EditorResourceLinkTreeNode",
		[dijit._TreeNode],
{
	/* 
	 * This is exactly the same as dijit._TreeNode 
	 * except the labelNode in this template is an anchor tag instead of a span
	 */
	templateString: dojo.cache("ram", "templates/EditorResourceLinkTreeNode.html"),
	
	editButton: null,
	insertButton: null,
	deleteButton: null,
	
	constructor: function() {
		this.msg = dojo.i18n.getLocalization("ram", "RelationshipTreeMessages");
	},
	
	postCreate:function () {
		this.inherited(arguments);
		this._createMenuBar();
	},
	_createMenuBar: function() {
		var t = this.tree.model.store.getValue(this.item, "jsObjType");
		if (t != "asset")
			return;
		
		var info = this.tree.model.store.getValue(this.item, "relationshipInfo");
		if (info && info.reverseRange===true)
			return;
		
		var pMenuBar = new dijit.Toolbar();
		
		 this.editButton = new dijit.form.Button({
            label: this.msg.buttonEditRelationship,
            iconClass: "dijitEditorIcon rowActionEdit",
            showLabel: false
        });
        pMenuBar.addChild(this.editButton);
		
        this.deleteButton = new dijit.form.Button({
            label: this.msg.buttonRemove,
            iconClass: "dijitEditorIcon rowActionRemove",
            showLabel: false
        });
        pMenuBar.addChild(this.deleteButton);
        
        pMenuBar.placeAt(this.menuBarWrapper);
	}
});

dojo.declare(
		"ram.relationshiptree._EditorRangeRelationshipTreeNode",
		[ram.relationshiptree._EditorResourceLinkTreeNode],
{
		templateString: dojo.cache("ram", "templates/ToolbarTreeNode.html")		
});

dojo.declare("ram._RangeRelationshipDialogContent", [dijit._Widget, dijit._Templated, dijit._Contained], {
	templateString: dojo.cache("ram", "relationshiptree/RangeRelationshipDialogContent.html"),
	widgetsInTemplate: true,
	
	item: null,
	model: null,
	type: "",
	
	constructor: function() {
		this.msg = dojo.i18n.getLocalization("ram", "RelationshipTreeMessages");
	},

	postCreate: function(){
		this.inherited(arguments);
		
		var store = this.model.store;
		
		var info = store.getValue(this.item, "relationshipInfo");
		
		if (info && info.range && info.range.lowestVersion) {
			this.lowestVersion.attr("value", info.range.lowestVersion);
			this.lowestVersionTypeSelect.value = info.range.lowestVersionType;
		}
		if (info && info.range && info.range.highestVersion) {
			this.highestVersion.attr("value", info.range.highestVersion);
			this.highestVersionTypeSelect.value = info.range.highestVersionType;
		}
		
		if (info && info.range && info.range.type && info.range.type == "All")
			dojo.attr(this.rangeTypeRadio_all, "checked", true);
		else
			dojo.attr(this.rangeTypeRadio_highest, "checked", true);
	},
	
	editNode: function() {
		var store = this.model.store;
		var info = store.getValue(this.item, 'relationshipInfo');
		
		if (!info.range) {
			info.range = new RelationshipRange();
		}
		
		var changed = false;
		var lowestVersion = this.lowestVersion.get('value');
		if (info.range.lowestVersion != lowestVersion) {
			info.range.lowestVersion = lowestVersion;
			changed = true;
		} 
		var lowestVersionType = dojo.attr(this.lowestVersionTypeSelect, 'value');
		if (info.range.lowestVersionType != lowestVersionType) {
			info.range.lowestVersionType = lowestVersionType;
			changed = true;
		} 
		var highestVersion = this.highestVersion.get('value');
		if (info.range.highestVersion != highestVersion) {
			info.range.highestVersion = highestVersion;
			changed = true;
		} 
		var highestVersionType = dojo.attr(this.highestVersionTypeSelect, 'value');
		if (info.range.highestVersionType != highestVersionType) {
			info.range.highestVersionType = highestVersionType;
			changed = true;
		}
		
		var currentType = info.range.type;
		if (this.rangeTypeRadio_highest.checked && currentType != this.rangeTypeRadio_highest.value) {
			info.range.type = this.rangeTypeRadio_highest.value;
			changed = true;
		} else if (this.rangeTypeRadio_all.checked && currentType != this.rangeTypeRadio_all.value) {
			info.range.type = this.rangeTypeRadio_all.value;
			changed = true;
		}
		
		if (!changed) {
			this.cancelEdit();
			return;
		}
		
		// Remove all children so when query and add assets in range, we don't have duplicates
		this.model.getChildren(this.item, dojo.hitch(this, function(children){
			dojo.forEach(children, dojo.hitch(this,  function(entry, i) {
				this.model.store.deleteItem(entry);
			 }));
		}));
		
		if (lowestVersion.length == 0 && highestVersion.length == 0) {
			this.cancelEdit(); 
			info.range = null;
			store.setValue(this.item, 'relationshipInfo', info);
			return; // Both version fields are blank, so don't bother querying
		} else {
			info.id = 0;
			info.assetId = 0;
			store.setValue(this.item, 'relationshipInfo', info);
		}
		
		var guid = store.getValue(this.item, 'guid');
		var query = new StringBuffer(getAbsoluteUrl('oslc/assets.json?oslc_asset.query=oslc_asset:guid="'), guid, '"');
		
		if (lowestVersion.length > 0) {
			this.lowestVersionTypeSelect.value == 'GREATER_THAN_OR_EQUAL_TO' ? query.append(' and oslc_asset:version>="', lowestVersion, '"') : query.append(' and oslc_asset:version>"', lowestVersion, '"');
		}
		
		if (highestVersion.length > 0) {
			this.highestVersionTypeSelect.value == 'LESS_THAN_OR_EQUAL_TO' ? query.append(' and oslc_asset:version<="', highestVersion, '"') : query.append(' and oslc_asset:version<"', highestVersion, '"');
		}
		query.append('/sort=-oslc_asset:version');
		
		dojo.xhrGet({
			url: query,
			handleAs: 'json',
			headers: {'Accept': 'application/json'},
			load: dojo.hitch(this, function(response) {
				var results = response.results;
				if (results.length > 0) {
					var currentType = this.model.store.getValue(this.item, 'relationshipInfo').range.type;
					if (currentType == 'All')
						for (var i = 0; i < results.length; i++) {
							var newItem = {
									guid: results[i].id['ras:GUID'],
									version: results[i].id['ras:version'], 
									name: results[i].title, 
									assetTypeUri: unescape(results[i].type.resource), 
									jsObjType: 'assetInRange'
								};
							this.model.newItem(newItem, this.item);
						}
					else {
						var newItem = {
								guid: results[0].id['ras:GUID'],
								version: results[0].id['ras:version'], 
								name: results[0].title, 
								assetTypeUri: unescape(results[0].type.resource), 
								jsObjType: 'assetInRange'
							};
						this.model.newItem(newItem, this.item);
					}
				}
			}),
			error: function(response, ioArgs) {
				console.error('HTTP status code: ', ioArgs.xhr.status);
				return response;
			}
		});
		
		this.cancelEdit();
	},
	
	cancelEdit: function() {
		this.getParent().hide();
	}
});

dojo.declare(
	"ram.relationshiptree.EditorRelationshipTree",
	[dijit.Tree],
{
	relationshipTypes: null,
	
	postMixInProperties: function(){
		this.inherited(arguments);
	},
	
	constructor: function() {
		this._messages = dojo.i18n.getLocalization("ram", "RelationshipTreeMessages");
	},

	postCreate: function(){
		this.inherited(arguments);
	},
	getLabel: function(/*dojo.data.Item*/ item){
		var type = this.model.store.getValue(item, "jsObjType");
		if (type=="asset") {
			var info = this.model.store.getValue(item, "relationshipInfo");
			if (info && info.range)
				return this._messages.nodeRangeRelaionship;
		}
		return this.model.getLabel(item);	// String
	},
	getIconClass: function(/*dojo.data.Item*/ item, /*Boolean*/ opened){
		// summary:
		//		Overridable function to return CSS class name to display icon
		// tags:
		//		extension
		if(item == this.model.root) {
			return (opened ? "customFolderOpenedIcon" : "customFolderClosedIcon");
		} else {
			var type = this.model.store.getValue(item, "jsObjType");
			if (type=="relationship") 
				return "relationshipIcon";
			else if (type=="constraint")
				return "constraintIcon";
			else if (type=="unconstrained")
				return "unconstrainedRelationshipIcon";
			else {
				var info = this.model.store.getValue(item, "relationshipInfo");
				if (info && info.range)
					return "rangeIcon";
				else
					return "assetIcon";
			}
		}
	},
	
	_createTreeNode: function(/*Object*/ args) {
		// TODO problem with this is that nodes aren't recreated when change.  So when an asset changes from being an asset to an range, this isn't called again
		
		if (args.indent == -1) {
			// Hack to see if its root.  Calling getValue(args.item... on this item was breaking code
			 return new dijit._TreeNode(args);
		}
		var item = args.item;
		
		var type = this.model.store.getValue(item, "jsObjType");
		if (type == "asset" || type == "assetInRange") {
			var info = this.model.store.getValue(item, "relationshipInfo");
			if (info && info.range) {
				return new ram.relationshiptree._EditorRangeRelationshipTreeNode(args)
			} else {
				var tnode = new ram.relationshiptree._EditorResourceLinkTreeNode(args);
				
				var guid = this.model.store.getValue(item, "guid");
				var version = this.model.store.getValue(item, "version");

				tnode.labelNode.innerHTML = dojo.string.substitute(this._messages.nodeAsset, {name: args.label, version: version});
				
				var uri = getAbsoluteUrl("oslc/assets/" + guid + "/" + version);
				var presentationUri = getAbsoluteUrl("assetDetail/generalDetails.jsp?guid="+guid+"&v="+version)
				
				var navLink = new jazz.ui.ResourceLink({
									uri: uri,
									presentationUri: presentationUri,
									lazyFetch: true
								}, tnode.labelNode);
		        return tnode;
			}
		} else if (type=="constraint") // || type=="unconstrained")  maybe get this to work
			return new ram.relationshiptree._EditorConstraintTreeNode(args);
		else 
			return new dijit._TreeNode(args);
    },
    onClick: function(/* dojo.data */ item, /*TreeNode*/ node, /*Event*/ evt){
		// summary:
		//		Callback when a tree node is clicked
		// tags:
		//		callback
    	var domElement = evt.target;
    	if (domElement.tagName == "A") {
			// User clicked a link
			if (domElement.getAttribute('onclick') == null) {
				if (domElement.getAttribute('href')) 
					document.location = domElement.getAttribute('href');
			}
			else domElement.onclick()
    	}
	},
	_onKeyPress: function(/*Event*/ e){
		if (e.charCode==dojo.keys.SPACE)
			dojo.stopEvent(e);
		else 
			this.inherited(arguments);
	},
	_onClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
		// summary:
		//		Translates click events into commands for the controller to process

		var domElement = e.target,
			isExpandoClick = (domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText)
			isEditClick = nodeWidget.editButton && (domElement == nodeWidget.editButton.valueNode || domElement == nodeWidget.rowActionEditText),
			isDeleteClick = nodeWidget.deleteButton && (domElement == nodeWidget.deleteButton.valueNode || domElement == nodeWidget.rowActionDeleteText),
			isInsertClick = nodeWidget.insertButton && (domElement == nodeWidget.insertButton.valueNode);
		
		if( (this.openOnClick && nodeWidget.isExpandable) || isExpandoClick ){
			// expando node was clicked, or label of a folder node was clicked; open it
			if(nodeWidget.isExpandable){
				this._onExpandoClick({node:nodeWidget});
			}
		} else if(isEditClick) {
			this._editNode(nodeWidget);
		} else if (isDeleteClick) {
			// Special case, stop the event before deleting the element, then return.
			dojo.stopEvent(e);
			this._deleteNode(nodeWidget.item);
			return;
		} else if (isInsertClick) {
			this._insertNode(nodeWidget);
		} else {
			this._publish("execute", { item: nodeWidget.item, node: nodeWidget, evt: e } );
			this.onClick(nodeWidget.item, nodeWidget, e);
			this.focusNode(nodeWidget);
		}
		dojo.stopEvent(e);
	},
	
	_editNode: function(/*TreeNode*/ nodeWidget) {
		var dialogContent = new ram._RangeRelationshipDialogContent({
			model: this.model,
			item: nodeWidget.item
		});
		
		var dialog = new ram.RamDialog({ id:"editDialog", title: this._messages.dialogTitleRange, helpId: 'racn0001'});
		dialog.attr("content", dialogContent);
		dialog.show();
		
		dojo.connect(dialog, "hide", dialog, function(e){dialog.destroyRecursive(false)})

	},
	
	_insertNode: function(/*TreeNode*/ nodeWidget) {
		var w = nodeWidget.getParent();
		while (this.model.store.getValue(w.item, 'jsObjType') != "relationship") {
			w = w.getParent();
		}
		var relationshipType = this.model.store.getValue(w.item, "type");
		
		var insertItem = nodeWidget.item;
		
		var dialog = dijit.byId('ram_assetSelectionDialog');
		if (dialog)
			dialog.destroy();
		dialog = new ram.RamDialog({title: this._messages.dialogTitleAddRelatedAsset, id: 'ram_assetSelectionDialog'});
		dojo.body().appendChild(dialog.domNode);
		dialog.startup();
		dialog.show();
		// need to wait for the dialog to appear before loading the iframe, otherwise IE throws a fit (see defect 87844)
		setTimeout(dojo.hitch(this, function() {
			var picker = dijit.byId('jazz_remoteChooserContainer');
			if (picker)
				picker.destroy();
			var url = getAbsoluteUrl('secure/delegated/assetSelection.faces');
			var assetType = this.model.store.getValue(insertItem, 'assetTypeName');
			if (assetType)
				url += '?filter=fType:' + assetType;
			picker = new jazz.ui.RemoteChooserContainer({
				id: 'jazz_remoteChooserContainer',
				remoteLocation: url, 
				onData: dojo.hitch(this, this._addRelatedAssets, relationshipType)
			});
			dojo.style(picker.domNode, 'width', '640px');
			dojo.style(picker.domNode, 'height', '480px');
			dialog.attr('content', picker.domNode);
		}, 500));
	},
	
	_addRelatedAssets: function(relationshipType, json) {
		var response = dojo.fromJson(json);
		var results = response['oslc:results'];
		if (results.length > 0) {
			for (var i=0; i<results.length; i++) {
				dojo.xhrGet({
					url: results[i]["rdf:resource"],
					handleAs: "json",
					headers: {"Accept": "application/json"},
					load: dojo.hitch(this, function() {
						var newItem = {"guid": arguments[0].id["ras:GUID"],"version": arguments[0].id["ras:version"], "name": arguments[0].title, "assetTypeUri": unescape(arguments[0].type.resource), "jsObjType":"asset", "relationshipInfo": {"id": 0, "stateId": 0, "toAssetId": 0, "range": null, "source": false, "reverseRange": false}};
						this.addRelatedAsset(relationshipType, newItem);
					}),
					// The ERROR function will be called in an error case.
					error: function(response, ioArgs) {
						console.error("HTTP status code: ", ioArgs.xhr.status);
						return response;
					}
				});
			}
			var dialog = dijit.byId('ram_assetSelectionDialog');
			if (dialog)
				dialog.hide();
		} else { 
			// dialog cancel button clicked
			var dialog = dijit.byId('ram_assetSelectionDialog');
			if (dialog)
				dialog.hide();
		}
	},
	
	addRelatedAsset: function(/*string*/ relationshipType, /*dojo.data.Item*/ item) {
		this.model.store.fetch({
			query: {jsObjType: "relationship", type: relationshipType},
			onComplete: dojo.hitch(this, this._addRelatedAsset, item),
			onError: this.fetchFailed,
			queryOptions: {deep: true}
		});
	},
	
	_addRelatedAsset: function(/*dojo.data.Item*/ item, items, request) {
		if (items.length>0) {
			// Should only be one item in items array
			var rel = items[0];
			var matchingConstraint = this._getMatchingConstraint(item, rel.constraints); // TODO instead of rel.constraints, should call getChildren
			if (matchingConstraint) 
				item = this.model.newItem(item, matchingConstraint);
			else {
				var c = {"name": this._messages.nodeUnconstrained, "type": 0, "jsObjType":"unconstrained", "relatedAssets": []};
				var constraintItem = this.model.newItem(c, rel);
				
				var assetItem = this.model.newItem(item, constraintItem);
			}
		} else {
			var rel = {"name": this._getRelationshipNameFromType(request.query.type), "type": request.query.type, "jsObjType":"relationship", "constraints": []};
			var relItem = this.model.newItem(rel, this.model.root);
			
			var c = {"name": this._messages.nodeUnconstrained, "type": 0, "jsObjType":"unconstrained", "relatedAssets": []};
			var constraintItem = this.model.newItem(c, relItem);
			
			var assetItem = this.model.newItem(item, constraintItem);
		}
	},
	
	_getMatchingConstraint: function(/*dojo.data.Item*/ item, constraints) {
		for (var i=0; i<constraints.length; i++) {
			var constraintUri = this.model.store.getValue(constraints[i], "assetTypeUri");
			var isConstraint = this.model.store.getValue(constraints[i], "constraint");
			if (isConstraint && constraintUri.length==0) {
				// An "any type" constraint -- Check its constraint children for a match
				var match = this._getMatchingConstraint(item, constraints[i].constraints);
				console.debug(match);
				if (match)
					return match;
				else 
					return constraints[i];
			} else if (isConstraint && item.assetTypeUri == constraintUri)
				return constraints[i];
			else if (!isConstraint) {
				// Went through all the real constraints (they come before any "Unconstrained" branches if there are any)
				// Now found that this last item is an "Unconstrained" item, so add asset here
				return constraints[i];
			}
		}
	},
	
	_fetchFailed: function(error, request) {
		console.debug(arguments);
	},
	
	_deleteNode: function(/*dojo.data.Item*/ item) {
		this.model.store.deleteItem(item);
		this.onDelete();
	},
	
	_getRelationshipNameFromType: function(relationshipType) {
		for (var i=0; i<this.relationshipTypes.length; i++) {
			if (this.relationshipTypes[i].type == relationshipType)
				return this.relationshipTypes[i].name;
		}
	}
	
});